<?php
if (! class_exists('SLPPower_Cron')) {

    /**
     * The cron job processing class.
     *
     * @package StoreLocatorPlus\SLPPower\Cron
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     * text-domain: slp-power
     *
     * @property-read   string              $action             The cron action to be performed.
     * @property        SLPPower            $addon
     * @property        SLPPower_Messages   $messages           The message stack for the current import operation.
     */
    class SLPPower_Cron extends SLPlus_BaseClass_Object {
        public $action;
        public $addon;
        public $messages;

        /**
         * This to do at startup.
         */
        function initialize() {
            $this->addon->create_object_schedule_messages();
            $this->messages = $this->addon->messages['schedule'];
            if ( isset( $this->action ) ) {
                $this->messages->add_message( sprintf( __( 'Cron action %s initialized.', 'slp-power' ) , $this->action ) );
                $this->process_action();
            }
        }

        /**
         * Import a location CSV file.
         *
         * @param string $mode  What mode to run in : see SLPPower_CSVImport execute_action
         */
        private function import_csv( $mode ) {
            $this->messages->add_message( __( 'Import CSV started.' , 'slp-power' ) );
            $this->addon->create_CSVLocationImporter();
            $this->addon->csvImporter->prepare_for_import();
            $this->addon->csvImporter->execute_action( $mode );
            if ( count($this->addon->csvImporter->processing_report) > 0 ) {
                foreach ( $this->addon->csvImporter->processing_report as $message ) {
                    $this->add_message( $message );
                }
            }
            $this->messages->add_message(__( 'Import CSV finished.', 'slp-power' )  );
            $this->addon->slplus->notifications->delete_all_notices();
        }

        /**
         * Process the cron action.
         */
        function process_action() {

            switch ( $this->action ) {
                case 'import_csv':
                    $this->import_csv( 'immediate' );
                    break;

                case 'process_csv':
                    $this->import_csv( 'recurring' );  // TODO - check this, this may need to be different.
                    break;

                default:
                    $this->messages->add_message( sprintf(__('Action %s is unsupported','slp-power') , $this->action ) );
                    break;
            }
        }

        /**
         * Schedule a one-time import.
         *
         * @param array $file_meta
         */
        function schedule_one_time_job( $file_meta ) {
            if ( empty( $this->addon->options['cron_import_timestamp'] ) ) {
                $this->addon->options['cron_import_timestamp'] = 'now';
                $timestamp = time();
            } else {
                $timestamp = strtotime( $this->addon->options['cron_import_timestamp'] );
            }

            $scheduled_without_problems = wp_schedule_single_event( $timestamp, 'cron_csv_import', array( 'import_csv', $file_meta ) );

            if ( $scheduled_without_problems !== false ) {
                $this->messages->add_message( sprintf( __( 'Scheduled a one-time import at %s (%s).', 'slp-power' ) , $this->addon->options['cron_import_timestamp'],  $timestamp ) );
            } else {
                $this->messages->add_message( sprintf( __( 'Could not a one-time import at %s (%s).', 'slp-power' ) , $this->addon->options['cron_import_timestamp'] , $timestamp ) );
            }
        }

        /**
         * Schedule a recurring cron job.
         *
         * @param array $file_meta
         */
        function schedule_recurring_job( $file_meta ) {
            if ( empty( $this->addon->options['cron_import_timestamp'] ) ) {
                $this->addon->options['cron_import_timestamp'] = 'now';
                $timestamp = time();
            } else {
                $timestamp = strtotime( $this->addon->options['cron_import_timestamp'] );
            }

            $scheduled_without_problems = wp_schedule_event( $timestamp, $this->addon->options['cron_import_recurrence'], 'cron_csv_import', array( 'import_csv', $file_meta ) );

            if ( $scheduled_without_problems !== false ) {
                $this->messages->add_message( sprintf( __( 'Scheduled a recurring %s import at %s (%s).', 'slp-power' ) , $this->addon->options['cron_import_recurrence'] , $this->addon->options['cron_import_timestamp'] ,$timestamp ) );
            } else {
                $this->messages->add_message( sprintf( __( 'Could not schedule a recurring %s import at %s (%s).', 'slp-power' ) , $this->addon->options['cron_import_recurrence'] , $this->addon->options['cron_import_timestamp'] ,$timestamp ) );
            }
        }
  }
}