<?php
if ( !class_exists( 'SLPPower_CSVImport' ) ) :

/**
 * CSV Import
 *
 * @package   StoreLocatorPlus\CSVImport
 * @author    Lance Cleveland <lance@charlestonsw.com>
 * @copyright 2013 - 2016 Charleston Software Associates, LLC
 *
 *
 * @property            SLPPower                     $addon                      This addon pack.
 * @property            boolean                      $adle_setting               Autodetect line endings setting.
 * @property            string[]                     $data                       The current CSV data array.
 * @property            string[]                     $fieldnames                 List of field names being processed.
 * @property            mixed                        $filehandle                 The CSV file handle.
 * @property            boolean                      $first_has_been_skipped     True if the first line has already been skipped.
 * @property-read       boolean                      $load_data
 * @property            int                          $maxcols                    What is the maximum data columns allowed for this CSV file?
 * @property            SLPPower_Messages            $messages                   The message stack for the current import operation.
 * @property-read       string                       $mode_for_current_record    What mode is the current record in 'add' || 'update' || 'skip' - set by internal processes
 * @property            string                       $output                     Output format 'html' or 'json'
 * @property            mixed[]                      $processing_counts
 * @property            string[]                     $processing_report          The processing report.
 * @property-read       bool                         $reported_too_many_fields   Did we already report a too many fields problem?
 * @property-read       boolean                      $skip_geocoding             Should geocoding be skipped?
 */
class SLPPower_CSVImport extends SLPlus_BaseClass_Object {
    public $addon;
    private $adle_setting;
    private $base_fieldnames = array(
        'id'        => 'sl_id',
        'store'     => 'sl_store',
        'address'   => 'sl_address',
        'address2'  => 'sl_address',
        'city'      => 'sl_city',
        'state'     => 'sl_state',
        'zip'       => 'sl_zip',
        'country'   => 'sl_country',
        'latitude'  => 'sl_latitude',
        'longitude' => 'sl_longitude',
    );
    public $filehandle;
    public $file_meta = NULL;
    private $data;
    private $fieldnames;
    private $first_has_been_skipped = false;
    private $load_data;
    private $maxcols;
    public $messages;
	private $mode_for_current_record;
    public $output = 'html';
    private $processing_counts;
    public $processing_report = array();
    private $reported_too_many_fields = false;
    private $skip_geocoding = false;

    /**
     * Things we do at startup.
     */
    function initialize() {
        $this->addon->create_object_import_messages();
	    $this->messages = $this->addon->messages['import'];
        $this->slplus->set_php_timeout();
        $this->skip_geocoding = $this->slplus->is_CheckTrue( $this->addon->options[ 'csv_skip_geocoding' ] );
        $this->load_data = $this->slplus->is_CheckTrue( $this->addon->options[ 'load_data' ] );

        add_filter( 'slp_csv_processing_messages', array( $this, 'filter_SetMessages' ) );
        add_action( 'slp_csv_processing', array( $this, 'process_each_csv_line' ) );
    }

    /**
     * Add sl_ to any base data fields (id, store, address, address2, etc.) that are in the location data.
     *
     * @param   array $location_data
     * @return  array   location data with 'sl_' attached to the keys that are base fields.
     */
    public function add_sl_to_base_fieldnames( $location_data ) {
        $base_location_intersect = array_intersect_key( $this->base_fieldnames, $location_data );
        foreach ( $base_location_intersect as $csv_field => $location_data_field ) {
            if ( !empty( $location_data[ $csv_field ] ) ) {
                $location_data[ $location_data_field ] = $location_data[ $csv_field ];
                unset( $location_data[ $csv_field ] );
            }
        }

        return $location_data;
    }

    /**
     * Process the file being imported.
     *
     * cron_csv_import action takes 2 parameters:
     * param 1: the action to perform
     * param 2: the params to be sent to the action processor
     *
     * @param mixed $mode
     */
    public function execute_action( $mode = NULL ) {
        if ( $mode === NULL ) {
            $mode = $this->addon->options[ 'cron_import_recurrence' ];
        }

        // Cron Job CSV Import Processing
        //
        switch ( $mode ) {

            // Immediate Processing
            //
            case 'immediate':
            case 'none'     :
                wp_clear_scheduled_hook( 'cron_csv_import', array( 'import_csv', $this->file_meta ) );

                if ( $this->addon->options[ 'load_data' ] ) {
                    $this->load_directly_into_mysql();
                } else {
                    $this->process_File();
                }
                break;

            // Load a CSV file in temp directory.
            //
            case 'load':
                $this->open_file_and_start_importing();
                break;

            // ASAP
            //
            case 'at'  :
                $this->addon->create_cron_object();
                $this->addon->cron->schedule_one_time_job( $this->file_meta );
                break;

            // Hourly, Twice Daily, Daily
            //
            case 'recurring':
            default     :
                $this->addon->create_cron_object();
                $this->addon->cron->schedule_recurring_job( $this->file_meta );
                break;
        }
    }

    /**
     * Allows WordPress to process csv file types
     *
     * @param array $existing_mimes
     * @return string
     */
    public function filter_AddMimeType( $existing_mimes = array() ) {
        $existing_mimes[ 'csv' ] = 'text/csv';

        return $existing_mimes;
    }

    /**
     * Set the process count output strings the users sees after an upload.
     *
     * @param string[] $message_array
     *
     * @return mixed[]
     */
    public function filter_SetMessages( $message_array ) {
        return array_merge(
            $message_array,
            array(
                'added'           => __( ' new locations added.', 'slp-power' ),
                'location_exists' => __( ' pre-existing locations skipped.', 'slp-power' ),
                'malformed'       => __( ' locations skipped due to malformed CSV data.', 'slp-power' ),
                'not_updated'     => __( ' locations did not need to be updated.', 'slp-power' ),
                'skipped'         => __( ' locations were skipped due to duplicate address information.', 'slp-power' ),
                'updated'         => __( ' locations were updated.', 'slp-power' ),
            )
        );
    }

    /**
     * Get a remote file and store it locally.
     *
     * @param $remote_file
     */
    private function get_remote_file( $remote_file ) {
        if ( $this->slplus->helper->webItemExists( $remote_file ) ) {
            $response = wp_remote_get( $remote_file, array( 'timeout' => 300 ) );

            // File opened without any issues.
            //
            if ( is_array( $response ) && isset( $response[ 'body' ] ) && !empty( $response[ 'body' ] ) ) {
                $ftp_file = $response[ 'body' ];
                $local_file = wp_tempnam();

                file_put_contents( $local_file, $ftp_file );

                $this->file_meta[ 'csvfile' ] = array(
                    'name'     => 'slp_locations.csv',
                    'type'     => 'text/csv',
                    'tmp_name' => $local_file,
                    'error'    => ( is_bool( $ftp_file ) ? '4' : '0' ),
                    'size'     => strlen( $ftp_file ),
                    'source'   => 'direct_url',
                );

                // Houston, we have a problem...
                //
            } else {
                $this->addon->create_cron_object();
                $this->addon->cron->messages->add_message( __( 'Could not fetch the remote file.', 'slp-power' ) );
            }
        }
    }

    /**
     * Is this a valid CSV File?
     *
     * @return bool
     */
    private function is_valid_csv_file() {

        // Is the file name set?  If not, exit.
        //
        if ( !isset( $this->file_meta[ 'csvfile' ] ) ) {
            $message = __( 'CSV File Meta not set.', 'slp-power' );
            $this->messages->add_message( $message );
            $this->slplus->notifications->add_notice( 'error', $message );

            return false;
        }


        // Is the file name set?  If not, exit.
        //
        if ( !isset( $this->file_meta[ 'csvfile' ][ 'name' ] ) || empty( $this->file_meta[ 'csvfile' ][ 'name' ] ) ) {
            $message = __( 'Import file name not set.', 'slp-power' );
            $this->messages->add_message( $message );
            $this->slplus->notifications->add_notice( 'error', $message );

            return false;
        }

        // Does the file have any content?  If not, exit.
        //
        if ( $this->file_meta[ 'csvfile' ][ 'size' ] <= 0 ) {
            switch ( $this->file_meta[ 'csvfile' ][ 'error' ] ) {
                case UPLOAD_ERR_INI_SIZE:
                    $message = __( 'Import file exceeds the upload_max_filesize in php.ini.', 'slp-power' );
                    break;

                case UPLOAD_ERR_PARTIAL:
                    $message = __( 'Import file was only partially loaded.', 'slp-power' );
                    break;

                case UPLOAD_ERR_NO_FILE:
                    $message = __( 'Import file seems to have gone missing.', 'slp-power' );
                    break;

                default:
                    $message = __( 'Import file is empty.', 'slp-power' );
                    break;
            }
            $this->messages->add_message( $message );
            $this->slplus->notifications->add_notice( 'error', $message );

            return false;
        }

        // Is the file CSV?  If not, exit.
        //
        $arr_file_type = wp_check_filetype( basename( $this->file_meta[ 'csvfile' ][ 'name' ] ), array( 'csv' => 'text/csv' ) );
        if ( $arr_file_type[ 'type' ] != 'text/csv' ) {
            $message =
                __( 'Uploaded file needs to be in CSV format.', 'slp-power' ) .
                sprintf( __( 'Type was %s.', 'slp-power' ), $arr_file_type[ 'type' ] );
            $this->messages->add_message( $message );
            $this->slplus->notifications->add_notice( 'error', $message );

            return false;
        }

        return true;
    }

    /**
     * Load basic CSV files directly into MYSQL with LOAD DATA.
     *
     * This is MUCH faster than CSV parsing but requires the CSV has no extended data
     * such as categories, etc.
     */
    private function load_directly_into_mysql() {

        if ( !$this->is_valid_csv_file() ) {
            return;
        }

        $new_file = $this->move_csv_to_slpdir();
        if ( empty( $new_file ) ) {
            return;
        }

        if ( !$this->open_csv_file( $new_file ) ) {
            return;
        }

        $this->set_field_names();
        fclose( $this->filehandle );


        $core_fieldnames = array_filter( $this->fieldnames, array( $this, 'return_base_fields_only' ) );
        $field_list = join( ',', $core_fieldnames );

        global $wpdb;
        $table_name = $wpdb->prefix . "store_locator";

        $load_data_sql =
            sprintf(
                "LOAD DATA LOCAL INFILE '%s' INTO TABLE %s  " .
                "FIELDS TERMINATED BY ','   " .
                "ENCLOSED BY '\"'           " .
                "ESCAPED BY '\\\\'           " .
                "%s " .
                "( %s )"
                ,
                $new_file,
                $table_name,
                'IGNORE 1 LINES',
                $field_list
            );
        $this->slplus->db->query( $load_data_sql );

        // Now geocode them.
        //
        if ( !$this->skip_geocoding ) {
            $this->addon->recode_all_uncoded_locations();
        }
    }

    /**
     * Look to see if incoming Identifier data is already in the extended data set.
     *
     * @param mixed[] $location_data
     *
     * @return mixed[] $location_data
     */
    public function match_identifier_field_on_import( $location_data ) {
        if ( isset( $location_data[ 'identifier' ] ) && !empty( $location_data[ 'identifier' ] ) ) {

            // Fetch sl_id from provided identifier.
            //
            $location_se_data = $this->slplus->database->get_Record(
                array( 'select_slid_from_extended_data', 'where_identifier_matches', ),
                array( $location_data[ 'identifier' ], )
            );

            // If there the select returned a valid data record object.
            //
            if ( is_array( $location_se_data ) && isset( $location_se_data[ 'sl_id' ] ) && !empty ( $location_se_data[ 'sl_id' ] ) ) {
                $location_data[ 'sl_id' ] = $location_se_data[ 'sl_id' ];
            } else {
            	$this->mode_for_current_record = 'add';
            }
        }

        return $location_data;
    }

    /**
     * Move the CSV File to a local directory.
     *
     * @return string
     */
    private function move_csv_to_slpdir() {

        // Check WordPress has an uploads directory.
        //
        if ( !is_dir( SLPLUS_UPLOADDIR ) ) {
            echo "<div class='updated fade'>" .
                sprintf(
                    __( 'WordPress upload directory %s is missing, check directory permissions.', 'slp-power' ),
                    SLPLUS_UPLOADDIR
                ) .
                '</div>';

            return NULL;
        }

        // Make the SLP CSV Upload Directory \
        //
        $updir = SLPLUS_UPLOADDIR . 'csv';
        if ( !is_dir( $updir ) ) {
            mkdir( $updir, 0755 );
        }

        $new_file = $updir . '/' . $this->file_meta[ 'csvfile' ][ 'name' ];

        // Move File -
        // If csvfile source is set to csv_file_url assume an http or ftp_get
        // direct to disk,
        //
        // otherwise
        //
        // Assume HTTP POST (browser direct) use move_uploaded_file
        //
        if (
            isset( $this->file_meta[ 'csvfile' ][ 'source' ] ) &&
            ( $this->file_meta[ 'csvfile' ][ 'source' ] === 'direct_url' )
        ) {
            if ( !rename( $this->file_meta[ 'csvfile' ][ 'tmp_name' ], $new_file ) ) {
                echo $this->slplus->helper->create_string_wp_setting_error_box(
                    __( 'Imported CSV file could not be renamed.', 'slp-power' ) .
                    sprintf( __( 'Possibly out of disk space while trying to rename to %s', 'slp-power' ), $new_file )
                );

                return '';
            }

        } else {
            if ( !move_uploaded_file( $this->file_meta[ 'csvfile' ][ 'tmp_name' ], $new_file ) ) {
                echo $this->slplus->helper->create_string_wp_setting_error_box(
                    __( 'Uploaded CSV file could not be moved.', 'slp-power' ),
                    sprintf( __( 'Check folder permissions for %s', 'slp-power' ), $new_file )
                );

                return '';
            }
        }

        return $new_file;
    }

    /**
     * Override this to add special processing that skips the data processing of the file.
     *
     * @return bool
     */
    private function ok_to_process_file() {
        return true;
    }

    /**
     * Open the CSV File and set the filehandle.
     *
     * @param  mixed   $filename
     *
     * @return bool
     */
    private function open_csv_file( $filename ) {

        // Line Endings
        //
        $this->adle_setting = ini_get( 'auto_detect_line_endings' );
        ini_set( 'auto_detect_line_endings', true );

        // Can the file be opened? If not, exit.
        //
        if ( ( $this->filehandle = fopen( $filename, "r" ) ) === FALSE ) {
            print "<div class='updated fade'>" .
                __( 'Could not open CSV file for processing.', 'slp-power' ) . '<br/>' .
                $filename .
                '</div>';
            ini_set( 'auto_detect_line_endings', $this->adle_setting );

            return false;
        }

        // Set first line processing flag.
        //
        $this->first_has_been_skipped = false;

        return true;
    }

    /**
     * Load the CSV file and start importing it into memory.
     */
    public function open_file_and_start_importing() {
        if ( !$this->open_csv_file( $this->file_meta[ 'local_file' ] ) ) {
            return;
        }

        $this->processing_report = array();

        $this->messages->add_message( '<hr/>', true );
        $this->messages->add_message( sprintf( __( 'Starting CSV import - duplicates %s .', 'slp-power' ) , $this->addon->options['csv_duplicates_handling'] ) );

        $this->set_field_names();

        // Reset the notification message to get a clean message stack.
        //
        $this->slplus->notifications->delete_all_notices();

        // Add CSV as a mime type
        //
        add_filter( 'upload_mimes', array( $this, 'filter_AddMimeType' ) );
        $reccount = 0;
        $this->maxcols = count( $this->fieldnames );

        $this->processing_counts = array(
            'added'       => 0,
            'exists'      => 0,
            'not_updated' => 0,
            'skipped'     => 0,
            'malformed'   => 0,
            'updated'     => 0,
        );

        // FILTER: slp_csv_processing_messages
        // Set the message array to be printed out for the above counters.
        //
        $location_processing_types = apply_filters( 'slp_csv_processing_messages', array() );

        // Turn off notifications for OK addresses.
        //
        $this->slplus->currentLocation->geocodeSkipOKNotices = true;
        $this->slplus->currentLocation->validate_fields = true;

        // Loop through all records
        //
        if ( $this->ok_to_process_file() ) {
            $this->messages->add_message( sprintf( __( 'Loading locations %s ', 'slp-power' ), $this->file_meta[ 'local_file' ] ), true );
            while ( ( $this->data = fgetcsv( $this->filehandle ) ) !== FALSE ) {
                $this->data = array_map( array( $this, 'strip_utf8_control_chars' ), $this->data );

                // Skip First Line
                //
                if ( !$this->first_has_been_skipped ) {
                    $this->first_has_been_skipped = true;
                    continue;
                }

	            $this->mode_for_current_record = $this->addon->options['csv_duplicates_handling'];

	            /**
	             * HOOK: slp_csv_processing
	             *
	             * The hook that processes each record of a CSV Import for the Power add on.
	             */
		        do_action( 'slp_csv_processing' );
                $reccount++;
            }
        }
        fclose( $this->filehandle );
        $this->messages->add_message( __( 'Finished CSV import.', 'slp-power' ) );

        $this->slplus->currentLocation->validate_fields = false;

        ini_set( 'auto_detect_line_endings', $this->adle_setting );

        // Processing Report
        //
        if ( $reccount > 0 ) {
            $this->processing_report[] = sprintf( __( '%d data lines read from the CSV file.', 'slp-power' ), $reccount );
        }
        foreach ( $this->processing_counts as $count_type => $count ) {
            if ( $count > 0 ) {
                $this->processing_report[] = sprintf( "%d %s", $count, $location_processing_types[ $count_type ] );
            }
        }

        if ( ( $this->output !== 'json' ) && ( count( $this->processing_report ) > 0 ) ) {
            foreach ( $this->processing_report as $message ) {
                printf( '<div class="updated fade">%s</div>', $message );
            }
        }

        /**
         * HOOK: slp_csv_processing_complete
         *
         * Run this action when the CSV import process has been completed.
         */
        do_action( 'slp_csv_processing_complete' );
    }

    /**
     * Things we do to prepare for an import.
     */
    public function prepare_for_import() {
        $this->addon->create_object_category_manager();

        add_filter( 'slp_csv_locationdata', array( $this, 'add_sl_to_base_fieldnames' ), 8 );
        add_filter( 'slp_csv_locationdata', array( $this, 'strip_extra_spaces_from_csv_location_data' ) );
        add_filter( 'slp_csv_locationdata', array( $this, 'match_identifier_field_on_import' ) );
        add_filter( 'slp_csv_locationdata', array( $this->addon->category_manager, 'create_categories_from_location_data' ), 30 );
        add_filter( 'slp_extend_get_SQL', array( $this, 'sql_match_identifier' ) );

        add_action( 'slp_location_added', array( $this, 'update_category_and_page_id' ) );

        do_action( 'slp_prepare_location_import' );

        if ( $this->slplus->is_CheckTrue( $this->addon->options[ 'csv_clear_messages_on_import' ] ) ) {
            $this->messages->clear_messages();
        }

        return $this->set_FileMeta();
    }

    /**
     * Process the lines of the CSV file.
     */
    public function process_each_csv_line() {
	    $num = count( $this->data );
	    if ( $num > $this->maxcols ) {
		    $this->report_malformed_csv_record( $num );

		    return;
	    }

	    // Set the locationData array with field names.
	    //
	    $locationData = array();
	    for ( $fldno = 0; $fldno < $num; $fldno ++ ) {
		    $locationData[ $this->fieldnames[ $fldno ] ] = $this->data[ $fldno ];
	    }

	    /**
	     * FILTER: slp_csv_locationdata
	     * Change location data before adding to database after read from CSV file.
	     *
	     * @params  array[]     $locationData
	     */
	    $locationData = apply_filters( 'slp_csv_locationdata', $locationData );

	    if ( ! isset( $locationData['sl_latitude'] ) ) {
		    $locationData['sl_latitude'] = '';
	    }
	    if ( ! isset( $locationData['sl_longitude'] ) ) {
		    $locationData['sl_longitude'] = '';
	    }

	    // Go add the CSV Data to the locations table.
	    //
	    $resultOfAdd = $this->slplus->currentLocation->add_to_database(
		    $locationData,
		    $this->mode_for_current_record,
		    $this->skip_geocoding ||
		    (
			    $this->slplus->currentLocation->is_valid_lat( $locationData['sl_latitude'] ) &&
			    $this->slplus->currentLocation->is_valid_lng( $locationData['sl_longitude'] )
		    )
	    );

        // Add the results of this location to the detailed message stack.
        //
	    $category_message = ! empty( $locationData['category'] ) ? ' [' . $locationData['category'] . ']' : '';
        $this->messages->add_message(
            "{$this->slplus->currentLocation->store}  {$resultOfAdd}{$category_message}"
        );



        // FILTER: slp_csv_locationdata_added
        // Post-location import processing.
        //
        // TODO: this should be do_action   change ER, EXP, PREMIER, PRO, SME to add_action.
        //
        apply_filters( 'slp_csv_locationdata_added', $locationData, $resultOfAdd );

        // Update processing counts.
        //
        $this->processing_counts[ $resultOfAdd ]++;
    }

    /**
     * Process a CSV File.
     */
    private function process_File() {
        if ( is_null( $this->file_meta ) ) {
            $this->file_meta = $_FILES;
        }

        if ( !$this->is_valid_csv_file() ) {
            return;
        }

        // Move file to temp directory.
        //
        $this->file_meta[ 'local_file' ] = $this->move_csv_to_slpdir();
        if ( empty( $this->file_meta[ 'local_file' ] ) ) {
            $this->messages->add_message( __( 'Local copy of CSV file could not be created.', 'slp-power' ) );

            return;
        }

        // Schedule Cron
        //
        $this->addon->create_cron_object();
        if ( wp_schedule_single_event( time(), 'cron_csv_import', array( 'process_csv', $this->file_meta ) ) === false ) {
            $this->messages->add_message(
                sprintf( __( 'Could not schedule the event to load the %s into the database.', 'slp-power' ), $this->file_meta[ 'csvfile' ][ 'name' ] )
            );

            return;
        }

        $this->messages->add_message( __( 'Starting the process of loading of the CSV file into the database.', 'slp-power' ) );

        // Start import process
        // TODO: move this to an AJAX process.
        //
	    $this->addon->csvImporter->messages->add_message(__( 'CSV Import Process File.', 'slp-power' ) );
        $this->open_file_and_start_importing();
    }

    /**
     * Report a malformed CSV record.
     *
     * @param  int $field_count
     */
    private function report_malformed_csv_record( $field_count ) {
        $this->processing_counts[ 'malformed' ]++;
        if ( !$this->reported_too_many_fields ) {
            $this->processing_report[] =
                __( 'The first line of your CSV file defines fields that do not match the data.', 'slp-power' ) . '<br/>' .
                sprintf( __( 'At least one line had %d fields while the header defined %d.', 'slp-power' ), $field_count, $this->maxcols ) . '<br/>' .
                __( 'Your header specified these fields: ', 'slp-power' ) . '<br/>' .
                join( ',', $this->fieldnames );
            $this->reported_too_many_fields = true;
        }
    }

    /**
     * Only use the base table fields.
     *
     * @param $var
     *
     * @return bool
     */
    function return_base_fields_only( $var ) {
        $var = preg_replace( '/^sl_/', '', $var );

        return ( array_search( $var, $this->slplus->currentLocation->dbFields, true ) !== false );
    }

    /**
     * Set the field names array for the fields being processed.
     *
     */
    function set_field_names() {

        // Try reading the first line of the file.
        //
        if ( ( $header_columns = fgetcsv( $this->filehandle ) ) !== FALSE ) {
            foreach ( $header_columns as $field_name ) {
                $this->fieldnames[] = sanitize_key( $field_name );
            }
            $this->first_has_been_skipped = true;
            $field_name_message = join( ', ', $this->fieldnames );
            $this->messages->add_message(
                __( 'Your CSV file header is set to import the following fields: ', 'slp-power' ) .
                $field_name_message,
                true
            );

            // Could not read first line of file.
            //
        } else {
            $this->messages->add_message(
                __( 'Could not read CSV file header. ', 'slp-power' ) .
                __( 'Is the file empty? ', 'slp-power' ),
                true
            );
        }
    }

    /**
     * Set file meta for the import.
     *
     * Use the standard browser file upload objects $_FILES if set.
     *
     * If not set check for a remote file URL and use that.
     *
     */
    private function set_FileMeta() {
        if ( !is_null( $this->file_meta ) ) {
            return NULL;
        }

        // Browser File Upload
        //
        if ( isset( $_FILES ) && ( !empty( $_FILES[ 'csvfile' ][ 'name' ] ) ) ) {
            $this->file_meta = $_FILES;

            return 'immediate';
        }

        // Remote File Set For Cron
        //
        if ( defined( 'DOING_CRON' ) ) {
            if ( empty( $this->addon->options[ 'csv_file_url' ] ) ) {
                $this->addon->cron->messages->add_message( __( 'File url is blank.', 'slp-power' ) );

                return NULL;
            }
            $remote_file = $this->addon->options[ 'csv_file_url' ];
            /** @noinspection PhpIncludeInspection */
            include_once( ABSPATH . 'wp-admin/includes/file.php' );

            // Remote File Direct Load
            //
        } else {
            if ( !isset( $_REQUEST[ 'slp-power' ] ) || !isset( $_REQUEST[ 'slp-power' ][ 'csv_file_url' ] ) || empty( $_REQUEST[ 'slp-power' ][ 'csv_file_url' ] ) ) {
                $this->messages->add_message( __( 'File url is blank.', 'slp-power' ) );

                return NULL;
            }
            $remote_file = $_REQUEST[ 'slp-power' ][ 'csv_file_url' ];
        }

        $this->get_remote_file( $remote_file );

        return NULL;
    }

    /**
     * Add the identifier filter to the SQL where clause.
     *
     * @param   string $sql_slug The SQL command slug
     *
     * @return  string              The SQL string snippet
     */
    public function sql_match_identifier( $sql_slug ) {
        if ( $sql_slug === 'where_identifier_matches' ) {
            return $this->slplus->database->add_where_clause( "identifier='%s'" );
        } else {
            return $sql_slug;
        }
    }

    /**
     * Start the CSV import of locations.
     */
    public function start_import() {
        $this->prepare_for_import();
        $this->process_File();
    }

    /**
     * Strip extra spaces from location data.
     *
     * @param $location_data
     *
     * @return string[] $location_data
     */
    public function strip_extra_spaces_from_csv_location_data( $location_data ) {
        return array_map( 'trim', $location_data );
    }

    /**
     * Strip UTF-8 control characters
     * @param string $string
     *
     * @return string
     */
    private function strip_utf8_control_chars( $string ) {
        return preg_replace( '/\p{Cc}/u', '', $string );
    }

    /**
     * Attach our category data to the update string.
     *
     * Put it in the sl_option_value field as a serialized string.
     *
     * Assumes currentLocation is set.
     *
     * @return string the modified SQL set syntax
     */
    public function update_category_and_page_id() {
        $this->addon->create_object_category_manager();
        $this->addon->category_manager->update_location_category();

    }
}

endif;