<?php
if ( !class_exists( 'SLPPower_Category_Manager' ) ) :

/**
 * Manage categories for admin, UI, CRON, AJAX, REST.
 *
 * @package   StoreLocatorPlus\Power\Categories
 * @author    Lance Cleveland <lance@charlestonsw.com>
 * @copyright 2016 Charleston Software Associates, LLC
 *
 * text-domain: slp-power
 *
 * @property        SLPPower                    $addon
 * @property        SLPPower_Category_Data      $category_data          The category table database interface.
 * @property        array                       $location_categories    The current location category ID array.
 * @property        SLPlus                      $slplus
 * @property        WP_Term[]                   $wp_categories
 * @property-read   int                         $wp-category-count      Count of active categories. Fetch with get_category_count()
 */
class SLPPower_Category_Manager extends SLPlus_BaseClass_Object {
    public $addon;
	public $wp_categories;
	private $wp_category_count;
    public $category_data;
    public $location_categories;

    /**
     * Things we do at the start.
     */
    public function initialize() {
        $this->addon->create_object_category_data();
        $this->category_data = $this->addon->category_data;
    }

	/**
	 * Add location counts to terms returned by WP get_term
	 *
	 * @param array         $terms      Array of found terms.
	 * @param array         $taxonomies An array of taxonomies.
	 * @param array         $args       An array of get_terms() arguments.
	 * @param WP_Term_Query|null $term_query The WP_Term_Query object.  (Only available in WP 4.6+, null otherwise)
	 *
	 * @return array
	 */
    public function add_location_counts_to_terms( $terms , $taxonomies , $args , $term_query = null) {
	    $this->addon->create_object_category_data();

	    foreach ( $terms as $key => $term ) {
		    $count_data = $this->addon->category_data->get_Record( 'select_count_for_termid' , $term->term_id );
		    $terms[$key]->location_count = $count_data['location_count'];
	    }

	    return $terms;
    }

    /**
     * Add single quotes to a string.
     *
     * @param string $string
     *
     * @return string
     */
    public function add_single_quotes( $string ) {
        return "'$string'";
    }

    /**
     * Additional processing of CSV records during an import.
     *
     * @param string[] $locationData
     *
     * @return string[]
     */
    public function create_categories_from_location_data( $locationData ) {

        // Allow "categories" as alias
        //
        if ( !isset( $locationData[ 'category' ] ) && isset( $locationData[ 'categories' ] ) ) {
            $locationData[ 'category' ] = $locationData[ 'categories' ];
        }

        // No category set, make it blank
        //
        if ( !isset( $locationData[ 'category' ] ) ) {
            $this->location_categories = array();

            return $locationData;
        }

        $locationData[ 'category' ] = trim( $locationData[ 'category' ] );
        if ( empty( $locationData[ 'category' ] ) ) {
            $this->location_categories = array();

            return $locationData;
        }

        // Build the category list
        //
        $this->location_categories = array();
        $locationData[ 'category' ] = wp_kses_decode_entities( trim( $locationData[ 'category' ] ) );
        $categories = explode( ',', $locationData[ 'category' ] );

        // If category could be located, attach it to the location.
        //
        foreach ( $categories as $categoryName ) {

            // Create category if necessary
            //
            $this->create_store_category( array( 'category' => $categoryName ) );

            // Manage colon separated category names,
            // assume they are in category import parent::child::grandchild::n-child format
            //
            $colon_separated_categories = explode( '::', $categoryName );

            // Process each of the separated colon category names
            //
            foreach ( $colon_separated_categories as $individual_category_name ) {
                $category = get_term_by( 'slug', sanitize_title( $individual_category_name ), SLPlus::locationTaxonomy );
                if ( is_object( $category ) && $category->term_id ) {
                    $this->location_categories[] = $category->term_id;
                }
            }
        }

        return $locationData;
    }

    /**
     * Create the store categories from the CSV data.
     *
     * categoryData elements include:
     * - category <string> the category name with :: to separate parents and children
     * - description <string> description of the category
     * - slug <string> which slug to use
     * - medium_icon <string> the URL to the icon file
     * - map_marker <string> the URL for the map marker file
     *
     * @param string[] $categoryData
     * @return string action taken
     */
    private function create_store_category( $categoryData ) {
        if ( empty( $categoryData[ 'category' ] ) ) {
            return;
        }

        $category_name_list = preg_split( '/::/', $categoryData[ 'category' ] );
        $category_count = count( $category_name_list );
        $new_category[ 'term_id' ] = 0;
        $current_category_level = 1;
        foreach ( $category_name_list as $category_name ) {

            // Is the the last category in the list (no preceding ::)
            //
            $is_last_category = ( $current_category_level++ === $category_count );

            // Setup extra category data like parent pointer, description, and slug
            //
            $extra_category_data = array();
            if ( $is_last_category && !empty( $categoryData[ 'slug' ] ) ) {
                $extra_category_data[ 'slug' ] = $categoryData[ 'slug' ];
            }
            $extra_category_data[ 'parent' ] = $new_category[ 'term_id' ];
            $extra_category_data[ 'description' ] =
                ( $is_last_category && !empty( $categoryData[ 'description' ] ) ) ?
                    $categoryData[ 'description' ] :
                    $category_name;

            // Add the new category if it does not exist.
            //
            $new_category = term_exists( $category_name, 'stores' );
            if ( !isset( $new_category[ 'term_id' ] ) || ( $new_category[ 'term_id' ] === 0 ) ) {
                $new_category =
                    wp_insert_term(
                        $category_name,
                        'stores',
                        $extra_category_data
                    );
            }
        }
    }

    /**
     * Remove any categories no longer attached to a location from the mapping table.
     */
    private function delete_detached_categories() {
        $inTerms = join( ',', array_map( array( $this, 'add_single_quotes' ), $this->location_categories ) );
        $this->category_data->db->query(
            $this->category_data->db->prepare( $this->category_data->get_SQL( 'delete_category_by_id' ), $this->slplus->currentLocation->id ) .
            ( !empty( $inTerms ) ? 'AND TERM_ID NOT IN (' . $inTerms . ')' : '' )
        );
    }

	/**
	 * Get all of the SLPlus::locationTaxonomy terms available.
	 */
    private function get_all_categories() {
    	if ( ! isset( $this->wp_categories ) ) {
    		add_filter( 'get_terms' , array( $this, 'add_location_counts_to_terms' ) , 10 , 4 );
		    $this->wp_categories = get_categories( array(
			    'taxonomy'   => SLPlus::locationTaxonomy,
			    'hide_empty' => false,
		    ) );
		    remove_filter( 'get_terms' , array( $this , 'add_location_counts_to_terms' ) , 10 );
	    }
    }

    /**
     * Set the location categories based on the $_REQUEST['tax_input'] values.
     */
    public function get_categories_from_input() {
        $this->set_categories_from_input();

        return $this->location_categories;
    }

	/**
	 * Return the number of WP categories available for locations.
	 * @return int
	 */
	public function get_category_count() {
		$this->get_all_categories();
		if ( ! isset( $this->wp_category_count ) ) {
			$this->wp_category_count = count( $this->wp_categories );
		}
		return $this->wp_category_count;
	}

    /**
     * Return the categories array in a SLP taxonomy notation for WordPress.
     * @return array
     */
    public function get_taxonomy() {
        return array( SLPlus::locationTaxonomy => $this->location_categories );
    }

    /**
     * Add the location_categories into a named array with key 'stores' for WP taxonomy processing.
     */
    public function get_taxonomy_from_input() {
        $this->set_categories_from_input();
        return $this->get_taxonomy();
    }

    /**
     * Map categories to their locations in the categories data table.
     *
     * Does NOT update the currentLocation (wp_store_locator table) attributes.
     */
    public function map_categories_to_locations() {
        if ( ! $this->slplus->currentLocation->isvalid_ID() ) {
        	return;
        }
        foreach ( $this->location_categories as $category ) {
            $this->category_data->add_RecordIfNeeded( $this->slplus->currentLocation->id, $category );
        }
    }

	/**
	 * Render the extended Tagalong fields on the WordPress add/edit category forms for store_page categories.
	 *
	 * @param   int     $term_id      the category id.
	 * @param   mixed[] $TagalongData tagalong data for this category.
	 */
	public function render_ExtraCategoryFields( $term_id = NULL, $TagalongData = NULL ) {
		$icon_img = SLPLUS_PLUGINURL . '/images/icons/blank.png';

		print '<div id="tagalong_extended_data">';
		print '<h3>Tagalong' . __( ' Extended Data', 'slp-power' ) . '</h3>';

		// Category Rank
		//
		print
			'<div class="form-field short_field">' .
			'<label for="rank">Rank</label>' .
			'<input type="text" id="rank" name="rank" value="' . $TagalongData[ 'rank' ] . '">' .
			'<p>' . __( 'Rank for category (1, 2, 3...) lower numbers have higher precedence for map markers.', 'slp-power' ) . '</p>' .
			'</div>';

		// Medium Icon
		//
		print
			'<div class="form-field icon_setting">' .
			'<label for="medium-icon">Medium Icon</label>' .
			'<input type="text" id="medium-icon" name="medium-icon" value="' . $TagalongData[ 'medium-icon' ] . '">' .
			'<p>' . __( 'This graphic appears as the icon or icon array in the result listing or map info bubble.', 'slp-power' ) . '</p>' .
			'<p><img id="medium-icon-image" align="top" src="' . $icon_img . '"></p>' .
			$this->show_Image( $TagalongData[ 'medium-icon' ] ) .
			'</div>';
		print $this->slplus->AdminUI->CreateIconSelector( 'medium-icon', 'medium-icon-image' );

		// Map Marker
		//
		print
			'<div class="form-field icon_setting">' .
			'<label for="map-marker">Map Marker</label>' .
			'<input type="text" id="map-marker" name="map-marker" value="' . $TagalongData[ 'map-marker' ] . '">' .
			'<p>' . __( 'This is the graphic used as the map marker on the map.', 'slp-power' ) . '</p>' .
			'<p><img id="map-marker-image" align="top" src="' . $icon_img . '"></p>' .
			$this->show_Image( $TagalongData[ 'map-marker' ] ) .
			'</div>';
		print $this->slplus->AdminUI->CreateIconSelector( 'map-marker', 'map-marker-image' );

		// Category URL
		//
		print
			'<div class="form-field">' .
			'<label for="category_url">Category URL</label>' .
			'<input type="text" id="category_url" name="category_url" value="' . $TagalongData[ 'category_url' ] . '">' .
			'<p>' . __( 'The URL where you want the icon to be linked.', 'slp-power' ) . '</p>' .
			'</div>';

		// URL Target
		//
		print
			'<div class="form-field">' .
			'<label for="url_target">URL Target</label>' .
			'<input type="text" id="url_target" name="url_target" value="' . $TagalongData[ 'url_target' ] . '">' .
			'<p>' . __( 'Target window for the URL (_blank = new window/tab, _self = same window/tab, csa = window/tab named "csa").', 'slp-power' ) . '</p>' .
			'</div>';

		print '</div>';
	}

    /**
     * Reset the current location store categories attributes.
     */
    private function reset_current_location_categories() {
        if ( !empty ( $this->slplus->currentLocation->attributes[ 'store_categories' ] ) ) {
            if ( is_array( $this->slplus->currentLocation->attributes[ 'store_categories' ] ) ) {
                unset( $this->slplus->currentLocation->attributes[ 'store_categories' ] );
            }
            $this->slplus->currentLocation->update_Attributes( array() );
        }
    }

    /**
     * Set the array of location category IDs passed in from $_REQUEST['tax_input']['stores']
     */
    public function set_categories_from_input() {
        if ( !isset( $this->location_categories ) ) {
            if (
                isset( $_REQUEST[ 'tax_input' ] ) &&
                isset( $_REQUEST[ 'tax_input' ][ SLPlus::locationTaxonomy ] ) &&
                is_array( $_REQUEST[ 'tax_input' ][ SLPlus::locationTaxonomy ] )
            ) {
                $this->location_categories = $_REQUEST[ 'tax_input' ][ SLPlus::locationTaxonomy ];
            } else {
                $this->location_categories = array();
            }
        }
    }

    /**
     * Set the store page taxonomy.
     *
     * @param   array $page_attributes
     *
     * @return array
     */
    public function set_page_taxonomy( $page_attributes ) {
        return array_merge( $page_attributes, array( 'tax_input' => $this->get_taxonomy() ) );
    }

	/**
	 * Return the div string to createstring_ShortcodeOutput an image.
	 *
	 * @param string $img - fully qualified image url
	 *
	 * @return string - the div text string with the image in it
	 */
	private function show_Image( $img = NULL ) {
		if ( is_null( $img ) ) {
			return '&nbsp;';
		}
		if ( empty( $img ) ) {
			return '&nbsp;';
		}

		return '<div class="slp_tagalog_category_image">' .
		       '<img src="' . $img . '"/>' .
		       '</div>';
	}

    /**
     * Attach our category data to the current location.
     *
     * @return string the modified SQL set syntax
     */
    function update_location_category() {

        add_filter( 'slp_location_page_attributes', array( $this, 'set_page_taxonomy' ) , 30 );

        $this->slplus->currentLocation->crupdate_Page( true );

        $this->delete_detached_categories();

        $this->reset_current_location_categories();

        $this->map_categories_to_locations();
    }

}

endif;
