<?php

if ( !class_exists( 'SLPPower_UI_Shortcode_SLP_Pages' ) ) {

    /**
     * UI pages stuff on loaded of using pages is enabled.
     *
     * @package StoreLocatorPlus\Power\UserInterface\Shortcode\slp_pages
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     * @property        SLPPower    $addon
     * @property-read   string[]    $categories_selected    Array of selected categories.
     * @property-read   int         $max_locations          Maximum number of locations to return.
     * @property-read   string      $order                  ASC or DESC
     * @property-read   string      $orderby                Field to sort pages list by.
     * @property-read   array       $page_retrieval_args    The args used to fetch the page list.
     * @property-read   string      $style                  Style of listing
     *
     */
    class SLPPower_UI_Shortcode_SLP_Pages extends SLPlus_BaseClass_Object {
        public $addon;
        private $categories_selected = array();
        private $max_locations = -1;
        private $order = 'ASC';
        private $orderby = null;
        private $page_retrieval_args;
        private $style = 'summary';

        /**
         * Create the list of Store Pages.
         *
         * @return string   the output
         */
        private function create_page_list() {
            switch ( $this->style ) {
                case 'list':
                case 'bullet':
                    return $this->create_bullet_list();

                case 'full':
                    return $this->create_full_list();

                case 'summary':
                    return $this->create_summary_list();
            }
        }

        /**
         * Style 'bullet' - creates a bullet list of location names.
         *
         * @return string
         */
        private function create_bullet_list() {
            $output =
                '<li><h1 class="slp_pages title"><a href="[storepage post=permalink]">[storepage post=post_title]</a></li>'
            ;
            return
                '<div class="slp_pages_list bullet"><ul>'.
                $this->create_string_location_list( $output) .
                '</ul></div>'
                ;

        }

        /**
         * Style 'full' - creates a full list of location data.
         *
         * @return string
         */
        private function create_full_list() {
            $output =
                '<h1 class="slp_pages title"><a href="[storepage post=permalink]">[storepage post=post_title]</a></h1>' .
                '[storepage post=post_content]'
                ;
            return
                '<div class="slp_pages_list full">'.
                $this->create_string_location_list( $output).
                '</div>'
                ;
        }

        /**
         * Style 'summary' - creates a summary list of location data.
         *
         * @return string
         */
        private function create_summary_list() {
            $output =
                '<h1 class="slp_pages title"><a href="[storepage post=permalink]">[storepage post=post_title]</a></h1>' .
                '<p  class="slp_pages address">[storepage field=address]</p>'
                ;
            return
                '<div class="slp_pages_list summary">'.
                $this->create_string_location_list( $output) .
                '</div>'
                ;
        }

        /**
         * Create the location list string.
         *
         * @param string $output    The output HTMl with shortcodes.
         *
         * @return string
         */
        private function create_string_location_list( $output ) {
            $this->set_page_retrieval_args();
            $page_list = new WP_Query( $this->page_retrieval_args );
            $html = '';
            if ( ! empty( $page_list ) ) {
                foreach ( $page_list->posts as $page ) {
                    $location_id = $this->slplus->database->get_Value( array( 'selectslid' , 'wherelinkedpostid' , 'limit_one') , $page->ID );
                    $this->slplus->currentLocation->set_PropertiesViaDB( $location_id );
                    $content = do_shortcode( $output );
                    $content = apply_filters( 'the_content' , $content );
                    $content = str_replace( ']]>', ']]&gt;', $content );

                    if ( ! empty( $this->addon->options[ 'pages_read_more_text' ] ) ) {
                        $extended_content = get_extended( $content );
                        $stripped_content = wp_strip_all_tags( $extended_content['extended'] , true );
                        if ( empty( $stripped_content ) ) {
                            $content = $extended_content['main'];
                        } else {
                            $content =
                                $extended_content['main'] .
                                sprintf( '<span class="slp_pages more"><a href="%s">%s</a></span>',
                                    get_permalink( $page->ID ),
                                    $this->addon->options[ 'pages_read_more_text' ]
                                );
                        }
                    }
                    $html .= sprintf(
                        '<div id="slp_page_location_%d" class="slp_page location_details">%s</div>',
                        $location_id,
                        $content
                    );
                }
            }

            return $html;
        }

        /**
         * Set filters on which pages to return.
         *
         * @param   string $category_value
         */
        private function filter_by_category( $category_value ) {
            $this->categories_selected = explode(',', $category_value);
        }

        /**
         * Manage the shortcode
         *
         * @param array $attributes {
         *      Named array of attributes set in shortcode.
         *
         *      @type string $category  Name of category to include.
         *      @type string $style     Listing style can be 'bullet' or 'summary'. Default 'summary'.
         * }
         *
         * @param string $content the existing content that we will modify
         *
         * @return string the modified HTML content
         */
        function process_shortcode( $attributes , $content = null ) {

            // Process the attributes
            //
            $field = null;
            $title = null;
            $attributes = (array) $attributes;
            foreach ( $attributes as $key => $value ) {
                $key = strtolower( $key );
                switch ( $key ) {

                    case 'category':
                        $this->filter_by_category( $value );
                        break;

                    case 'max_locations':
                        $this->set_max_locations( $value );
                        break;

                    case 'order':
                        $this->set_order( $value );
                        break;

                    case 'orderby':
                    case 'order_by':
                        $this->set_orderby( $value );
                        break;


                    case 'style':
                        if ( ! empty( $value ) ) {
                            $this->style = $value;
                        }
                        break;
                }
            }

            $content = $this->create_page_list();

            return $content;
        }

        /**
         * Set the maximum locations to return.
         *
         * @param int $max_locations
         */
        private function set_max_locations( $max_locations ) {
            $max_locations = (int) $max_locations;
            $this->max_locations = ( $max_locations < 1 ) ? -1 : $max_locations;
        }

        /**
         * ASC/DESC sort order of the pages.
         *
         * @param string $order   a* = ascending , d* = descending
         */
        private function set_order( $order ) {
            $this->order = substr( strtolower( $order ) , 0 ) === 'a' ? 'ASC' : 'DESC';
        }

        /**
         * Field to sort the pages by.
         *
         * @param string $orderby   a* = ascending , d* = descending
         */
        private function set_orderby( $orderby ) {
            $this->orderby = strtolower( $orderby );
            if ( substr( $this->orderby, 0, 4 ) === 'rand') {
                $this->orderby = 'rand';
            }
        }

        /**
         * Set the page retrieval arguments.
         */
        private function set_page_retrieval_args() {
            if ( ! isset( $this->page_retrieval_args ) ) {
                $this->page_retrieval_args = array(
                    'echo'      => false,
                    'post_type' => SLPLus::locationPostType,
                    'title_li'  => '',
                    'nopaging'  => ( $this->max_locations < 1 ),
                    'order'     => $this->order,
                    'posts_per_page'    => $this->max_locations,
                );

                if ( ! is_null( $this->orderby ) ) {
                    $this->page_retrieval_args['orderby'] = $this->orderby;
                }

                if ( ! empty ( $this->categories_selected ) ) {
                    $this->page_retrieval_args['tax_query'] = array(  array(
                        'taxonomy' => SLPlus::locationTaxonomy,
                        'field'    => 'name' ,
                        'terms'    => $this->categories_selected
                        )
                    );
                }

            }
        }
    }

}