/**
 * A base class that helps add-on packs separate ui functionality.
 *
 * Add on packs should include and extend this class.
 *
 * This allows the main plugin to only include this file when rendering the front end.
 * Reduces the front-end footprint.
 *
 * Uses the module pattern.
 * @link https://en.wikipedia.org/wiki/Module_pattern
 * @link https://toddmotto.com/mastering-the-module-pattern/
 *
 * @package StoreLocatorPlus\Premier\UserInterfaceJS
 * @author Lance Cleveland <lance@storelocatorplus.com>
 * @copyright 2015 - 2016 Charleston Software Associates, LLC
 *
 * @type {SLPPREMIER}
 */


// Setup the Premier namespace
var SLPPREMIER = SLPPREMIER || {};

/**
 * The Location List Class
 */
SLPPREMIER.location_list = ( function () {

    var current_page = 0; // Which results page are we on?

    /**
     * Add pagination to the location output.
     */
    this.add_pagination = function( ) {


        if ( !cslmap.latest_response) { return; }

        // Create page div
        //
        var div = document.createElement('div');
        div.innerHTML = cslmap.latest_response.premier.pagination_block;

        // Attach to sidebar
        //
        var sidebar = document.getElementById('map_sidebar');
        sidebar.insertBefore( div, sidebar.firstChild);

        // Add on click elements
        //
    };

    /**
     * Add a header to results output.
     *
     * @param results_html
     */
    this.add_results_header = function ( results_html ) {
        slp_Filter('replace_shortcodes_options').subscribe(SLPPREMIER.utils.use_our_options);
        results_html.content = slppremier_settings.results_header.replace_shortcodes( {} );
        if ( results_html.content.indexOf('add_locations_here') >= 0 ) {
            results_html.insert_rows_at = '#add_locations_here';
        }
        slp_Filter('replace_shortcodes_options').unsubscribe(SLPPREMIER.utils.use_our_options);
    };

    /**
     * Submit next page of locations request.
     */
    this.get_next_page = function () {
        current_page++;
        map_center = cslmap.lastCenter;
        jQuery.post(
            slplus.ajaxurl ,
            {
                lat: map_center.lat(),
                lng: map_center.lng(),
                radius: cslmap.lastRadius,
                action: 'csl_ajax_onload',
                page: current_page
            },
            cslmap.process_ajax_response
        )
    };

    /**
     * Submit previous page of locations request.
     */
    this.get_previous_page = function () {
        current_page--;
        map_center = cslmap.lastCenter;
        jQuery.post(
            slplus.ajaxurl ,
            {
                lat: map_center.lat(),
                lng: map_center.lng(),
                radius: cslmap.lastRadius,
                action: 'csl_ajax_onload',
                page: current_page
            },
            cslmap.process_ajax_response
        )
    };

    /**
     * Mark location result strings as finished to prevent wrapping in a div.
     *
     * @param results_html
     */
    this.mark_results_finished = function ( results_html ) {
        results_html.finished = true;
    };

    /**
     * Subscribe to the slp.js location string related filters.
     *
     * Subscribe to geocoder_request
     * Subscribe to geocode_results
     *
     * @see https://api.jquery.com/jQuery.Callbacks/
     */
    this.setup_location_list_subscriptions = function () {

        // Do Not Wrap Results Enabled
        if ( slppremier_settings.results_no_wrapper === '1' ) {
            slp_Filter('wrap_location_results').subscribe(mark_results_finished);
        }

        // Results Header
        //
        if ( slppremier_settings.results_header ) {
            slp_Filter('location_results_header').subscribe(add_results_header);
        }

        // Pagination Enabled
        //
        if ( slppremier_settings.pagination_enabled === '1' ) {
            jQuery('#map_sidebar').on('contentchanged', add_pagination);
        }
    };

    /**
     * Public methods.
     */
    return {
        setup_location_list_subscriptions: setup_location_list_subscriptions
    };
} ) ();

/**
 * The Premier Location Search Class
 */
SLPPREMIER.location_search = ( function () {

    /**
     * Subscribe to the slp.js geocoding related filters.
     *
     * Subscribe to geocoder_request
     * Subscribe to geocode_results
     *
     * @see https://api.jquery.com/jQuery.Callbacks/
     */
    this.setup_geocoding_subscriptions = function () {

        // Country Influence Enabled
        if ( slppremier_settings.region_influence_enabled === '1' ) {
            slp_Filter('geocoder_request').subscribe(add_region_to_geocoder_request);
        }

        // Locations Influence Enabled
        if ( slppremier_settings.boundaries_influence_type !== 'none' ) {
            slp_Filter('geocoder_request').subscribe(add_location_bounds_to_geocoder_request);
            slp_Filter('geocode_results').subscribe(set_best_result);
        }

        // Show Address Guess Enabled
        //
        if ( slppremier_settings.show_address_guess          === '1' ) {
            slp_Filter('geocode_results').subscribe(show_google_address_guess);
        }
    }

    /**
     * Extend the geocoder request to add bounds to influence the google address lookup.
     *
     * Uses the location data set in SLP to set the sw/ne corners of a bounding box to
     * influence how Google guesses which address smoeone means when they type in text
     * into an address search box.
     *
     * @param {google.maps.GeocoderRequest} geocoder_request
     */
    this.add_location_bounds_to_geocoder_request = function( geocoder_request) {
        var _this = this;
        if (slppremier_settings.bounds) {
            var sw_bounds = new google.maps.LatLng( slppremier_settings.bounds.min_lat , slppremier_settings.bounds.min_lng );
            var ne_bounds = new google.maps.LatLng( slppremier_settings.bounds.max_lat , slppremier_settings.bounds.max_lng );
            geocoder_request['bounds'] = new google.maps.LatLngBounds( sw_bounds, ne_bounds);
        }
    }

    /**
     * Extend the geocoder request to add region to influence the google address lookup.
     *
     * @param {google.maps.GeocoderRequest} geocoder_request
     */
    this.add_region_to_geocoder_request = function( geocoder_request) {
        var _this = this;
        if ( slppremier_settings.region ) {
            geocoder_request['region'] = slppremier_settings.region;
        }
    }

    /**
     * Find the first Google guessed address within the lat/lng bounds set by add_location_bounds_to_geocoder_request.
     *
     * @param geocode_results
     */
    this.set_best_result = function( geocode_results ) {
        if ( ! geocode_results.request.bounds ) { return; }

        if (slppremier_settings.bounds) {
            var total_results = geocode_results.all.length;
            for ( var result_entry = 0 ; result_entry < total_results ; result_entry++ ) {
                if ( geocode_results.request.bounds.contains( geocode_results.all[result_entry].geometry.location ) ) {
                    geocode_results.best = geocode_results.all[result_entry];
                    return;
                }
            }
        }
    }

    /**
     * Show the best guess address to the user, replacing what they typed in the address box.
     *
     * @param geocode_results
     */
    this.show_google_address_guess = function( geocode_results ) {
        if ( jQuery('#addressInput').val() ) {
            jQuery('#addressInput').val(geocode_results.best.formatted_address);
        }
    }

    /**
     * Auto submit dropdowns.
     */
    this.autosubmit_dropdowns = function() {
        cslmap.loadMarkers( cslmap.lastCenter , cslmap.lastRadius , cslmap.saneValue('tag_to_search_for','') );
    };

    /**
     * Public methods.
     */
    return {
        setup_geocoding_subscriptions: setup_geocoding_subscriptions,
        autosubmit_dropdowns: autosubmit_dropdowns
    }
} ) ();

/**
 * Marker Cluster Class.
 */
SLPPREMIER.marker_cluster = ( function () {
    var last_active_marker;
    var all_markers;
    var cluster_options = {
        gridSize: parseInt(slplus.options.cluster_gridsize),
        minimumClusterSize: parseInt(slplus.options.cluster_minimum),
    };

    /**
     * Create the clusters.
     */
    this.create = function () {
        var gmlist = [];
        for (idx = 0 ; idx < cslmap.markers.length; idx++ ) {
            gmlist.push( cslmap.markers[idx].__gmarker );
        }
        all_markers = new MarkerClusterer( cslmap.gmap , gmlist , cluster_options );
    };

    /**
     * Get all the clustered markers.
     * @returns {*}
     */
    this.get_clustered_markers = function () {
        return all_markers;
    }

    /**
     * Public methods.
     */
    return {
        create: create,
        all_markers: get_clustered_markers
    }
} ) ();

/**
 * The Premier Google Map Manager
 */
SLPPREMIER.map = ( function () {

    /**
     * Subscribe to the slp.js map options filters.
     *
     * Subscribe to map_options
     *
     * @see https://api.jquery.com/jQuery.Callbacks/
     */
    this.setup_map_subscriptions = function () {
        slp_Filter('map_options').subscribe(modify_map_options);
        if ( slplus.options.map_marker_tooltip === '0' ) {
            slp_Filter('marker').subscribe(modify_marker);
        }

        // Bubble Footnote Set
        //
        if ( slppremier_settings.bubble_footnote ) {
            slp_Filter('replace_shortcodes_options').subscribe(SLPPREMIER.utils.use_our_options);
        }
    };

    /**
     * Hide the StreetView on the Google Map interface.
     *
     * @param map_options
     */
    this.modify_map_options = function( map_options ) {
        map_options.streetViewControl = ( slppremier_settings.map_option_hide_streetview !== '1' );
        map_options.clickableIcons = ( slplus.options.map_options_clickableIcons === '1' );
    };

    /**
     * Hide the StreetView on the Google Map interface.
     *
     * @param map_options
     */
    this.modify_marker = function( marker ) {
        marker.__title = '';
    };

    /**
     * Public methods.
     */
    return {
        setup_map_subscriptions: setup_map_subscriptions
    }
} ) ();

/**
 * Premier Results Manager
 */
SLPPREMIER.results = ( function() {
    var last_active_marker;
    var last_marker_meta = {};
    var that = this;
    var shade_listener;

    /**
     * Initialize
     */
    this.initialize = function () {
            jQuery('*').on('result_clicked', interact_with_marker );
    };

    /**
     * Handle All Marker Interaction
     * This allows us to control the order for fine control over the UX.
     */
    this.interact_with_marker = function( obj, event ) {
        var map_marker = event.data.marker.__gmarker;

        // Same marker we just were modifying - do nothing.
        if (map_marker === that.last_active_marker) {
            return;
        }

        var not_first_marker = ( typeof that.last_active_marker !== 'undefined' );

        var marker_meta = {
            map: map_marker.getMap(),
            icon: map_marker.getIcon(),
            label: map_marker.getLabel(),
            zindex: map_marker.getZIndex()
        };

        // Cluster Is Active
        //
        if (slplus.options.clusters_enabled === '1') {
            if (not_first_marker) {
                that.last_active_marker.setMap(that.last_marker_meta.map);
            }
            map_marker.setMap(cslmap.gmap);
        }

        // Move Map
        //
        if (slplus.options.results_click_map_movement === 'center') {
            cslmap.gmap.setCenter(map_marker.getPosition());
        }

        // Set Icon
        if (slplus.options.results_click_marker_icon_behavior !== 'as_is') {
            if (not_first_marker) {
                that.last_active_marker.setIcon(that.last_marker_meta.icon);
            }
            map_marker.setIcon(slplus.options.results_click_marker_icon);
        }

        // Animate
        //
        if (slplus.options.results_click_animate_marker !== 'none') {
            map_marker.setAnimation(google.maps.Animation.DROP);
        }

        // Label
        //
        if (slplus.options.results_click_label_marker !== 'no_label') {
            if (not_first_marker) {
                that.last_active_marker.setLabel(that.last_marker_meta.label);
            }
            var label_text = jQuery('<div/>').html(event.data.info[slplus.options.results_click_label_marker]).text();
            that.shade_listener = google.maps.event.addDomListener( cslmap.gmap , 'idle' , function() {
                var target_div = jQuery('div').filter( function() { var decoded = this.innerHTML.replace( /&amp;/g, '&'); return decoded === label_text; } );
                jQuery(target_div).css( 'margin-top' , '5em' );
                jQuery(target_div).css( 'background' , 'rgba(255,255,255,0.7)' );
                jQuery(target_div).css( 'padding' , '0.2em' );
                jQuery(target_div).css( 'border' , 'solid 1px black' );
                jQuery(target_div).css( 'border-radius' , '4px' );
                google.maps.event.removeListener( that.shade_listener );
            }
            );
            map_marker.setLabel(label_text);
        }

        // ZIndex
        //
        if (not_first_marker) {
            that.last_active_marker.setZIndex(that.last_marker_meta.zindex);
        }
        map_marker.setZIndex(500);

        that.last_active_marker = map_marker;
        that.last_marker_meta = marker_meta;
    };

    /**
     * Public Methods and Such...
     */
    return {
        initialize: initialize
    };
} )();

/**
 * Helper Utilities.
 */
SLPPREMIER.utils = ( function() {

    // Are location list features in play?
    //
    this.wants_to_extend_results = function() {
        return (
            ( slplus.options.results_click_map_movement          === 'center'  ) ||
            ( slplus.options.results_click_animate_marker       !== 'none'     ) ||
            ( slplus.options.results_click_marker_icon_behavior !== 'as_is'    ) ||
            ( slplus.options.results_click_label_marker         !== 'no_label' )
        );
    };

    /**
     * Return true if our settings want results interaction.
     */
    this.wants_results_interaction = function() {

    };

    // Are search features in play?
    //
    this.wants_to_extend_search = function() {
        return (
            ( slppremier_settings.boundaries_influence_type !== 'none' ) ||
            ( slppremier_settings.show_address_guess        === '1'    ) ||
            ( slppremier_settings.dropdown_autosubmit       === '1'    )
        );
    };

    /**
     * Do we need to the goecoding subscriptions?
     *
     * @returns {boolean}
     */
    this.needs_geocoding_subscription = function() {
        return (
            ( slppremier_settings.boundaries_influence_type !== 'none' ) ||
            ( slppremier_settings.show_address_guess        === '1'    )
        );
    };

    /**
     * Swap SLP options with Premier Options for shortcode replace.
     *
     * @param options
     */
    this.use_our_options = function( options ) {
        jQuery.extend( options , slppremier_settings );
    };

    /**
     * Public methods and vars.
     */
    return {
        wants_to_extend_results: wants_to_extend_results,
        wants_to_extend_search: wants_to_extend_search,
        needs_geocoding_subscription: needs_geocoding_subscription,
        use_our_options: use_our_options
    };
} ) ();

// Document Ready
jQuery( document ).ready(
    function () {

        // SelectMenu Drop Down Styler
        //
        if ( slppremier_settings.dropdown_style !== 'none' ) {
            jQuery('#sl_div select').selectmenu();
        }

        // Address Search Management
        //
        if ( SLPPREMIER.utils.wants_to_extend_search() ) {
            if ( SLPPREMIER.utils.needs_geocoding_subscription() ) {
                SLPPREMIER.location_search.setup_geocoding_subscriptions();
            }

            if ( slppremier_settings.dropdown_autosubmit === '1' ) {
                if ( slppremier_settings.dropdown_style !== 'none' ) {
                    jQuery('#sl_div select').on('selectmenuchange', SLPPREMIER.location_search.autosubmit_dropdowns);
                } else {
                    jQuery('#sl_div select').on('change', SLPPREMIER.location_search.autosubmit_dropdowns);
                }
            }
        }

        // Location List Management
        //
        if ( SLPPREMIER.utils.wants_to_extend_results() ) {
            SLPPREMIER.location_list.setup_location_list_subscriptions();
        }


        // Map Marker Management
        //
        if ( slplus.options.clusters_enabled === '1' ) {
            jQuery('#map').on('markers_dropped', SLPPREMIER.marker_cluster.create );
        }

        // Results Interactions
        //
        if ( SLPPREMIER.utils.wants_to_extend_results() ) {
            SLPPREMIER.results.initialize();
        }

        // Google Map Modifier
        //
        SLPPREMIER.map.setup_map_subscriptions();

    }
);
